# -*- coding: utf-8 -*-
# @Time : 2022/3/25 14:11
# @Author : Zhan Yong
from ccdt.dataset.base_labelme.base_labelme import BaseLabelme, PathTest, Encoder, json, collections, np
from pycocotools.coco import COCO
from tqdm import *
import os


class Coco(BaseLabelme):
    """
    coco实现类，主要实现coco转labelme，labelme转coco，同时继承BaseLabelme
    子类就可以访问到基类（父类）的属性和方法了，它提高了代码的可扩展性和重用行。
    """

    def __init__(self, *args, **kwargs):
        # 注释ID
        self.cur_ann_id = 0
        # 图片ID
        self.cur_image_id = 0
        # 图片对象列表
        self.images = list()
        # 注释对象列表
        self.annotations = list()
        # 类别对象列表
        self.categories = list()
        # 所有标注属性对象列表
        self.coco_shapes = list()
        # 目标检测类别对象列表
        self.categories_name = list()
        # 自定义coco文件名称
        self.coco_file_name = 'coco.json'
        self.coco_file = kwargs.get('coco_file')
        self.coco = None
        if self.coco_file:
            self.coco = COCO(self.coco_file)  # 加载coco文件数据
        super().__init__(*args, **kwargs)

    def self2labelme(self):
        """
        coco转labelme实现
        """
        # 如果self.coco为空，则实现labelme转coco
        if self.coco is None:
            return
        # 获取每一张图片的id
        img_ids = self.coco.getImgIds()
        # 循环遍历每一个id
        i = 0
        for img_id in tqdm(img_ids):
            img_info = self.coco.loadImgs(img_id)[0]
            obj_dir = PathTest.initialize(img_info['file_name'])
            relative_path = os.path.join('..', '00.images', img_info['file_name'])
            # 通过注释id寻找到同一帧图片下的多个注释属性
            labelme_data = self.single_coco2labelme(img_id)
            labelme_data['imageHeight'] = img_info['height']
            labelme_data['imageWidth'] = img_info['width']
            labelme_data['imagePath'] = relative_path
            data_info = dict(image_dir=self.images_dir,
                             image_file=img_info['file_name'],
                             labelme_dir=self.labelme_dir,
                             labelme_file=obj_dir.stem + '.json',
                             labelme_info=labelme_data,
                             data_type=self.data_type,
                             input_dir=self.input_dir,
                             output_dir=self.output_dir,
                             only_annotation=self.only_annotation)
            self.data_infos.append(data_info)
            i += 1
        print("coco转labelme结束,一共%d" % i)

    def single_coco2labelme(self, img_id):
        """
        实现每一个coco注释合成labelme注释
        @param img_id:
        @return:
        """
        ann_ids = self.coco.getAnnIds(imgIds=img_id)
        labelme_data = dict(
            version='4.5.9',
            flags={},
            shapes=[],
            imagePath=None,
            imageData=None,
            imageHeight=None,
            imageWidth=None
        )
        if ann_ids:
            shapes = []
            anns = self.coco.loadAnns(ann_ids)
            for ann in anns:
                # 取到同一张图片的多个注释属性
                category_id = ann['category_id']
                # 获取坐标框
                bbox = ann['bbox']
                # 坐标切割
                bbox = [bbox[0:2], bbox[2:4]]
                # 左上角的坐标(x,y)右上角的坐标(x,y+h)左下角的坐标(x+w,y)右下角的坐标(x+w,y+h)
                points = [bbox[0], [bbox[0][0] + bbox[1][0], bbox[0][1] + bbox[1][1]]]
                # 通过类别id获取类别名称
                cats = self.coco.loadCats(category_id)[0]
                # 取到类别名称
                name = cats['name']
                shape = {"label": name, "points": points, "group_id": None, "shape_type": "rectangle", "flags": {}}
                shapes.append(shape)
            labelme_data['shapes'] = shapes
        return labelme_data

    def self2coco(self):
        """
        labelme转coco实现
        """
        # 如果self.coco不为空，则实现coco转labelme
        if self.coco:
            return
        self.get_data_paths()
        self.load_labelme()
        self.update_property()
        if self.data_infos:
            # 定义coco数据格式
            coco_data = dict(
                images=self.images,
                annotations=self.annotations,
                categories=self.categories,
            )
            for labelme_info in self.data_infos:
                # 针对背景类直接跳过、有json文件没有图片文件直接跳过，不写入图片属性
                if labelme_info['labelme_info'] is None or labelme_info['labelme_info']['shapes'] == [] \
                        or labelme_info['image_file'] is None:
                    continue
                self.cur_image_id += 1
                images = dict(
                    file_name=labelme_info['image_file'],
                    height=labelme_info['labelme_info']['imageHeight'],
                    width=labelme_info['labelme_info']['imageWidth'],
                    id=self.cur_image_id,
                )
                self.images.append(images)
                self.coco_shapes = labelme_info['labelme_info']['shapes']
                # 标签属性处理
                one_img_ann_list = self.labelme_shapes2coco_ann()
                self.annotations.extend(one_img_ann_list)
            for category_id, category_name in enumerate(self.categories_name, 1):
                category = dict(
                    supercategory='object',
                    id=category_id,
                    name=category_name,
                )
                self.categories.append(category)
            obj_dir = PathTest.initialize(self.images_dir)
            # self.save_labelme()
            out_put_coco_file = obj_dir.joinpath(self.input_dir, obj_dir.parent, self.coco_file_name)
            with open(out_put_coco_file, 'w', encoding='utf-8') as coco_fp:
                json.dump(coco_data, coco_fp, ensure_ascii=False, indent=4, cls=Encoder)
        return

    def labelme_shapes2coco_ann(self):
        """
        把labeme标签转成coco标签
        @return:
        """
        shapes_type = collections.defaultdict(list)
        for shape in self.coco_shapes:
            if shape['shape_type'] == 'rectangle':
                if shape['label'] not in self.categories_name:
                    self.categories_name.append(shape['label'])
                shapes_type[shape['shape_type']].append(shape)
            elif shape['shape_type'] == 'polygon':
                continue
            elif shape['shape_type'] == 'line':
                continue
            elif shape['shape_type'] == 'linestrip':
                continue
            elif shape['shape_type'] == 'circle':
                continue
        one_img_ann_list = list()
        for shape_type, shapes_type_ann in shapes_type.items():
            if shape_type == 'rectangle':
                ann_list = self.rectangle_shapes2coco(shapes_type.get(shape_type))
                one_img_ann_list.extend(ann_list)
            elif shape_type == 'polygon':
                continue
            elif shape_type == 'line':
                continue
            elif shape_type == 'linestrip':
                continue
            elif shape_type == 'circle':
                continue
        return one_img_ann_list

    def rectangle_shapes2coco(self, shapes):
        """
        实现矩形框转换计算
        @param shapes:
        @return:
        """
        ann_list = []
        for shape in shapes:
            ann = self.get_default_ann()
            category_id = shapes.index(shape) + 1
            ann['category_id'] = category_id
            points = np.array(shape['points'])
            point_min, point_max = points.min(axis=0), points.max(axis=0)
            w, h = point_max - point_min
            ann['bbox'] = [point_min[0], point_min[1], w, h]
            ann['area'] = w * h
            ann_list.append(ann)
        return ann_list

    def get_default_ann(self):
        """
        标签属性定义，并id自增
        @return:
        """
        self.cur_ann_id += 1
        annotation = dict(
            segmentation=[],
            area=0,
            iscrowd=0,
            image_id=self.cur_image_id,
            bbox=[],
            category_id=0,
            id=self.cur_ann_id,
        )
        return annotation



# -*- coding: utf-8 -*-
# @Time : 2022/3/25 13:44
# @Author : Zhan Yong
# from .path_operate import PathOperate
from .encoder import Encoder
from .show_tool import ShowTool
from .path_test import PathTest

__all__ = ['Encoder', 'ShowTool', 'PathTest']



# -*- coding: utf-8 -*-
# @Time : 2022/3/30 11:35
# @Author : Zhan Yong
import prettytable as pt
from collections import Iterable


class TotalLabelmeInfos(object):
    """
    这里耗时太久，针对标签数量达到365个的时候，打印会很耗时
    """
    def __init__(self, shapes_data):
        self.obj_judge_condition = 0  # 数据集判断条件，单数据集为2，多数据集为1
        self.shapes_data = list()
        self.title = ''
        self.num_labelme = 0
        self.num_images = 0
        self.num_background = 0
        if isinstance(shapes_data, Iterable):  # 判断传入对象是否为可迭代对象
            self.shapes_data = shapes_data
            self.obj_judge_condition = 1
        else:
            self.shapes_data.append(shapes_data)
            self.obj_judge_condition = 2
        self.number = 0
        self.shapes_data_info = dict()

    def __iter__(self):
        return self

    def __next__(self):
        """
        每次迭代出想要的内容，又能抛出异常终止迭代，raise StopIteration
        #  https://www.cnblogs.com/xifengmo/p/11029391.html
        :return:
        """
        # cursor = 0
        # while cursor < len(self.shapes_data):
        #     yield self.shapes_data.items[cursor]
        #     cursor += 1
        # if self.number <= self.number:

        if self.number < self.shapes_data.__len__():
            for shapes_data in self.shapes_data:
                if self.obj_judge_condition == 1:
                    self.title = str(shapes_data.input_dir)
                    self.num(shapes_data.num_labelme, shapes_data.num_images, shapes_data.num_background)
                elif self.obj_judge_condition == 2:
                    # from ccdt.dataset.utils.path_operate import PathOperate
                    from ccdt.dataset import PathTest
                    title_path = PathTest.initialize(shapes_data.input_dir)
                    slash = str(title_path.joinpath(shapes_data.input_dir, shapes_data.images_dir))
                    self.title = slash.replace('\\', '/')  # 对象转字符串时，有转义字符，替换后方便，直接输入路径
                    self.num(shapes_data.num_labelme, shapes_data.num_images, shapes_data.num_background)
                for label_key, label_infos in shapes_data.label2label_info.items():
                    self.label_info(label_key, label_infos)
                self.number += 1
            return self.shapes_data_info
        else:
            raise StopIteration  # 抛出异常停止遍历

    def label_info(self, label_key, label_info):
        """
        封装同一个标签类别对应的相关属性为字典，方便打印输出
        封装后数据结构：{{'person': {'shape_type': ['rectangle'], 'flags': ['狗狗', '活', '安全帽', '工装'], 'group_id': [None]}}
        :param label_key: 标签类别
        :param label_info: 标签类别对应相关属性
        """
        label_shape_flag_group = dict()
        label_shape_flag_group[label_key] = {'shape_type': [], 'flags': [], 'group_id': []}
        shape_type = list()
        group_id = list()
        flags = list()
        for label in label_info:
            shape_type += label[0]
            flags += label[1]
            group_id += label[2]
            label_shape_flag_group[label_key]['shape_type'] = list(set(shape_type))
            label_shape_flag_group[label_key]['group_id'] = list(set(group_id))
        label_shape_flag_group[label_key]['flags'] = list(set(self.flags_info(flags)))
        return self.shapes_data_info.update(label_shape_flag_group)

    def flags_info(self, flags):
        """
        封装类别标签属性flags，把同一个标签类别下flags属性值为True的键封装成列表返回
        封装前数据结构：<class 'list'>: [{'活': False}, {'狗狗': True}, {'工装': False}, {'安全帽': True}]
        封装后数据结构：['狗狗','工装']
        :param flags:
        :return:
        """
        fg = list()
        for flag in flags:
            for key, value in flag.items():
                if flag[key] is True:
                    fg.append(key)
        return fg

    def num(self, num_labelme, num_images, num_background):
        """
        对象图片数量、json文件数量、背景图片数量统计
        :param num_background:
        :param num_images:
        :param num_labelme:
        """
        self.num_labelme += num_labelme
        self.num_images += num_images
        self.num_background += num_background

    def __repr__(self):
        property_tb = pt.PrettyTable(['label_name', 'shape_type_name', 'flags_name'])
        for key, value in self.shapes_data_info.items():
            property_tb.add_row([key, value['shape_type'], value['flags']])
        # print(property_tb.get_string(title=self.title))
        num_tb = pt.PrettyTable(['num_images', 'num_labelme', 'num_background'])
        num_tb.add_row([self.num_images, self.num_labelme, self.num_background])
        print(num_tb.get_string(title=self.title))
        return str(property_tb.get_string(title=self.title))



# -*- coding: utf-8 -*-
# @Time : 2022/4/2 20:11
# @Author : Zhan Yong
import os
from pathlib import Path


class PathTest(object):
    def __init__(self, *args, **kwargs):
        """
        初始化方法
        :param args: 元组类型取值顺序，跟传参顺序有关
        :param kwargs:
        """
        self.dir_name = args[0]
        self.object_type = args[1]
        self.file_formats = args[2]
        self.coco_file = args[3]
        self.function = args[4]
        self.output_format = args[5]
        if self.function == 'merge':
            self.dir_paths = self.get_dir_paths()
        elif self.function == 'convert' and self.output_format == 'labelme':
            self.dir_paths = self.get_coco_to_labelme_paths()
        elif self.output_format == 'coco':
            self.dir_paths = self.get_dir_currency()
        else:
            self.dir_paths = self.get_dir_currency()

    def get_dir_paths(self):
        """
        递归组合，合并功能目录
        只循环一次，取第一层dirs目录
        :return:
        """
        path_name = list()  # 文件夹路径
        for root, dirs, files in os.walk(self.dir_name, topdown=True):
            dirs.sort()
            files.sort()
            if self.object_type == 'BaseLabelme':
                for dir_name in dirs:
                    input_data_format = dict(
                        type='',
                        images_dir='',
                        labelme_dir='',
                        file_formats='',
                        coco_file='',
                        input_dir=''
                    )
                    input_data_format['file_formats'] = self.file_formats
                    input_data_format['coco_file'] = self.coco_file
                    input_data_format['input_dir'] = self.dir_name
                    input_data_format['type'] = self.object_type
                    input_data_format['labelme_dir'] = dir_name
                    input_data_format['images_dir'] = dir_name
                    path_name.append(input_data_format)
                # 第一次循环结束就必须return
                return path_name

    @classmethod
    def get_merge_paths(cls, dir_name, file_formats, input_dir, output_dir, data_type, only_annotation):
        """
        递归组合，类别合并功能图片路径，把每一个类别的所有图片路径封装为一个对象。
        以图片为基准组合json文件索引目录
        :param dir_name:
        :param file_formats:
        :param input_dir:
        :param output_dir:
        :param data_type:
        :param only_annotation:
        :return:
        """
        data_paths = list()
        for root, dirs, files in os.walk(os.path.join(input_dir, dir_name), topdown=True):
            # 对目录和文件进行升序排序
            dirs.sort()
            files.sort()
            # 遍历文件名称列表
            if '01.labelme' in root:
                continue
            for file in files:
                obj_file = PathTest.initialize(file)
                if obj_file.suffix in file_formats:
                    make_image_dir = root.replace(os.path.join(input_dir, dir_name), '').strip('\\/')
                    make_labelme_dir = str(PathTest.initialize(make_image_dir).parent.joinpath('01.labelme'))
                    data_path = dict(image_dir=make_image_dir,
                                     image_file=file,
                                     labelme_dir=make_labelme_dir,
                                     labelme_file=obj_file.stem + '.json',
                                     input_dir=os.path.join(input_dir, dir_name),
                                     output_dir=output_dir,
                                     data_type=data_type,
                                     labelme_info=None,
                                     background=False,
                                     only_annotation=only_annotation, )
                    data_paths.append(data_path)
        return data_paths

    @classmethod
    def initialize(cls, images_dir):
        """
        路径实列对象初始化
        :param images_dir:
        :return:
        """
        obj_dir = Path(images_dir)
        return obj_dir

    def get_dir_currency(self):
        """
        通用递归路径组合，包含筛选功能、重命名功能、抠图功能、转换功能
        :return:
        """
        path_name = list()  # 文件夹路径
        for root, dirs, files in os.walk(self.dir_name, topdown=True):
            print()
            dirs.sort()
            files.sort()
            input_data_format = dict(
                type='',
                images_dir='',
                labelme_dir='',
                file_formats='',
                coco_file='',
                input_dir=''
            )
            for dir_name in dirs:
                # 存在的问题：如果约定目录（00.images与01.labelme）有变动，会导致数据加载不完整
                if dir_name == '00.images' or dir_name == '01.labelme':
                    replace_path = root.replace(self.dir_name, '').strip('\\/')
                    input_data_format['file_formats'] = self.file_formats
                    input_data_format['coco_file'] = self.coco_file
                    input_data_format['input_dir'] = self.dir_name
                    input_data_format['type'] = self.object_type
                    input_data_format['labelme_dir'] = os.path.join(replace_path, '01.labelme')
                    input_data_format['images_dir'] = os.path.join(replace_path, '00.images')
            if input_data_format.get('labelme_dir') != '' and input_data_format.get('images_dir') != '':
                # print(input_data_format.get('labelme_dir'))
                # print(input_data_format.get('images_dir'))
                # if input_data_format.get('labelme_dir') not in dirs:
                #     assert False, '{} 约定目录不为01.labelme'.format(root)
                # if input_data_format.get('images_dir') not in dirs:
                #     assert False, '{} 约定目录不为00.images'.format(root)
                path_name.append(input_data_format)
        return path_name

    @classmethod
    def get_filter_paths(cls, images_dir, file_formats, input_dir, output_dir, data_type, only_annotation, labelme_dir):
        """
        递归组合，筛选功能、重命名功能、抠图功能、的图片路径
        以图片为基准组合json文件索引路径
        :param images_dir:
        :param file_formats:
        :param input_dir:
        :param output_dir:
        :param data_type:
        :param only_annotation:
        :param labelme_dir:
        :return:
        """
        data_paths = list()
        for root, dirs, files in os.walk(os.path.join(input_dir, images_dir), topdown=True):
            # 对目录和文件进行升序排序
            dirs.sort()
            files.sort()
            # 遍历文件名称列表
            if '01.labelme' in root:
                continue
            for file in files:
                obj_file = PathTest.initialize(file)
                if obj_file.suffix in file_formats:
                    data_path = dict(image_dir=images_dir,  #
                                     image_file=file,
                                     labelme_dir=labelme_dir,
                                     labelme_file=obj_file.stem + '.json',
                                     input_dir=input_dir,  #
                                     output_dir=output_dir,
                                     data_type=data_type,
                                     labelme_info=None,
                                     background=False,
                                     only_annotation=only_annotation, )
                    data_paths.append(data_path)
        return data_paths

    @classmethod
    def get_videos_path(cls, root_dir, file_formats):
        """
        视频帧提取组合路径
        :param root_dir:
        :param file_formats:
        :return:
        """
        # dir_name_list = list()  # 文件夹名称
        # files_name = list()  # 文件名称
        # path_name = list()  # 文件夹路径
        file_path_name = list()  # 文件路径
        for root, dirs, files in os.walk(root_dir, topdown=True):
            dirs.sort()
            files.sort()
            # for dir_name in dirs:
            # if dir_name != '00.images' and dir_name != '01.labelme':
            # dir_name_list.append(dir_name)
            # 遍历文件名称列表
            for file in files:
                # 获取文件后缀
                file_suffix = os.path.splitext(file)[-1]
                # 如果读取的文件后缀，在指定的后缀列表中，则返回真继续往下执行
                if file_suffix in file_formats:
                    # 如果文件在文件列表中，则返回真继续往下执行
                    # files_name.append(file)
                    # path_name.append(root)
                    file_path_name.append(os.path.join(root, file))
        return file_path_name

    def get_coco_to_labelme_paths(self):
        """
        组合coco转labelme路径
        :return:
        """
        path_name = list()  # 文件夹路径
        for root, dirs, files in os.walk(self.dir_name, topdown=True):
            obj_path = Path(root)
            image_path = '00.images'
            labelme_path = '01.labelme'
            dirs.sort()
            files.sort()
            if self.object_type == 'Coco':
                input_data_format = dict(
                    type='',
                    images_dir='',
                    labelme_dir='',
                    file_formats='',
                    coco_file='',
                    input_dir=''
                )
                input_data_format['type'] = self.object_type
                input_data_format['input_dir'] = obj_path.parent.parent
                two_levels_path = os.path.join(root.split('/')[-2], root.split('/')[-1])
                input_data_format['images_dir'] = os.path.join(two_levels_path, image_path)
                input_data_format['labelme_dir'] = os.path.join(two_levels_path, labelme_path)
                input_data_format['coco_file'] = self.coco_file
                input_data_format['file_formats'] = self.file_formats
                path_name.append(input_data_format)
                # 第一次循环结束就必须return
                return path_name



# -*- coding: utf-8 -*-
# @Time : 2022/3/25 14:08
# @Author : Zhan Yong
from ccdt.dataset.utils import Encoder, PathTest
import shutil
import json
import os


class AsyncIoTask(object):
    """
    异步协程IO处理实现类
    参考网址：https://zhuanlan.zhihu.com/p/59621713
    """

    @staticmethod
    async def json_dump(data_info):
        """
        异步协程IO：写json数据到磁盘，拷贝图片数据到磁盘
        :param data_info:
        """
        copy_path = ''
        json_path = ''
        # print(f'get test for {data_info} times')
        # print(f'get test for {out_dir} times')
        jpg = PathTest.initialize(data_info['image_file'])
        # 如果图片名称后缀格式重复多次，就进行重写json文件后保存，重命名图片名称
        if data_info['image_file'].count(jpg.suffix) >= 2:
            # print(data_info['input_dir'])
            # print(data_info['image_dir'])
            # print(data_info['labelme_file'])
            # print(data_info['labelme_info'])
            # print('数据存在问题，路径如下')
            image_file = os.path.join(data_info['input_dir'], data_info['image_dir'], data_info['image_file'])
            if data_info['labelme_file']:
                labelme_file = os.path.join(data_info['input_dir'], data_info['image_dir'], data_info['labelme_file'])
                print(labelme_file)
            print(image_file)
            # print(old_image_path)
            # if data_info['labelme_info']:
            #     labelme_info = data_info['labelme_info']['imagePath'].replace(jpg.suffix, '')
            #     image_file = data_info['image_file'].replace(jpg.suffix, '')
            #     rewrite_labelme_info = labelme_info + jpg.suffix
            #     rewrite_image_file = image_file + jpg.suffix
            #     labelme_file = data_info['labelme_file'].replace(jpg.suffix, '')
            #     data_info['image_file'] = rewrite_image_file
            #     data_info['labelme_file'] = labelme_file
            #     data_info['labelme_info']['imagePath'] = rewrite_labelme_info
            #     new_image_path = os.path.join(data_info['input_dir'], data_info['image_dir'], rewrite_image_file)
            #     # 重命名图片名称
            #     os.rename(old_image_path, new_image_path)
            # else:
            #     # 只重命名图片名称
            #     image_file = data_info['image_file'].replace(jpg.suffix, '')
            #     rewrite_image_file = image_file + jpg.suffix
            #     new_image_path = os.path.join(data_info['input_dir'], data_info['image_dir'], rewrite_image_file)
            #     os.rename(old_image_path, new_image_path)
            # print(old_image_path)
        # 组合保存图片的文件夹路径
        save_images_dir = os.path.join(data_info['output_dir'], data_info['image_dir'])
        os.makedirs(save_images_dir, exist_ok=True)
        # 组合保存json的文件夹路径
        save_labelme_dir = os.path.join(data_info['output_dir'], data_info['labelme_dir'])
        os.makedirs(save_labelme_dir, exist_ok=True)
        # 组合输出json文件路径。其中replace("\\", "/")是为了解决OSError: [Errno 22] Invalid argument错误
        if data_info['labelme_file']:
            if str(jpg) == data_info['image_file']:
                json_path = os.path.join(save_labelme_dir, data_info['labelme_file']).replace("\\", "/")
        # 组合拷贝被拷贝图片的路径
        if data_info['data_type'] == 'Coco':
            if str(jpg) == data_info['image_file']:
                coco_image_dir = PathTest.initialize(data_info['image_dir']).parent
                copy_path = os.path.join(data_info['input_dir'], coco_image_dir, data_info['image_file'])
            else:
                # 针对特殊目录手动修改并组合
                coco_image_dir = PathTest.initialize(data_info['image_dir']).parent
                self_path = data_info['image_file'].split('/')
                copy_path = os.path.join(data_info['input_dir'], coco_image_dir, self_path[2], '00.images',
                                         self_path[3])
                # 修改labelme_dir和image_dir
                data_info['image_dir'] = os.path.join(os.path.dirname(data_info['image_dir']), self_path[2],
                                                      os.path.basename(data_info['image_dir']))
                data_info['labelme_dir'] = os.path.join(os.path.dirname(data_info['labelme_dir']), self_path[2],
                                                        os.path.basename(data_info['labelme_dir']))
                # 同时修改json路径imagesPath的相对路径
                data_info['labelme_info']['imagePath'] = os.path.join('../', '00.images', self_path[3]).replace('\\',
                                                                                                                '/')
                # 组合保存图片的文件夹路径
                save_images_dir = os.path.join(data_info['output_dir'], data_info['image_dir'])
                os.makedirs(save_images_dir, exist_ok=True)
                # 组合保存json的文件夹路径
                save_labelme_dir = os.path.join(data_info['output_dir'], data_info['labelme_dir'])
                os.makedirs(save_labelme_dir, exist_ok=True)
                json_path = os.path.join(save_labelme_dir, data_info['labelme_file']).replace("\\", "/")
        if data_info['data_type'] == 'BaseLabelme':
            copy_path = os.path.join(data_info['input_dir'], data_info['image_dir'], data_info['image_file'])
        if data_info['only_annotation'] is False:
            try:
                shutil.copy(copy_path, save_images_dir)  # shutil.SameFileError,当同一文件夹拷贝相同文件会出错
            except Exception as e:
                print(e)
                print(copy_path)
                print(save_images_dir)
        try:
            with open(json_path, "w", encoding='UTF-8', ) as labelme_fp:  # 以写入模式打开这个文件
                json.dump(data_info['labelme_info'], labelme_fp, indent=4, cls=Encoder)  # 从新写入这个文件，把之前的覆盖掉（保存）
        except Exception as e:
            if 'No such file or directory' in e.args:
                pass
            else:
                print(e)
                print(json_path)
                print(data_info)



# -*- coding: utf-8 -*-
# @Time : 2022/2/18 17:49
# @Author : Zhan Yong
from setuptools import find_packages
from setuptools import setup
import io
with io.open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()


def get_install_requires():
    install_requires = [
        'tqdm',
        'opencv_python',  # for PyInstaller
        'numpy',
        'pycocotools',
        'prettytable',
    ]
    return install_requires


setup(
    # 取名不能够用_会自动变-   ccdt
    name='ccdt',
    version='1.0.31',
    packages=find_packages(exclude=['data']),
    install_requires=get_install_requires(),
    author='zhanyong',
    author_email='zhan.yong@chipeak.com',
    description='AI数据转换工具箱',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/chipeak/chipeak_cv_data_tool',
    project_urls={
        'Bug Tracker': 'https://github.com/chipeak/chipeak_cv_data_tool/issues',
    },
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
    ],

    # package_data={'cpdt': ['icons/*', 'config/*.yaml']},
    entry_points={
        'console_scripts': [
            'ccdt=ccdt.dataset.__main__:main',
            # 视频切片集成
             'video=ccdt.video_tool.__main__:main',
            # 数据分配，分配图片，分配labelme
            #  'video=',
            # 'file',
            # 'labelme=labelme.__main__:main',
            # 'labelme_draw_json=labelme.cli.draw_json:main',
            # 'labelme_draw_label_png=labelme.cli.draw_label_png:main',
            # 'labelme_json_to_dataset=labelme.cli.json_to_dataset:main',
            # 'labelme_on_docker=labelme.cli.on_docker:main',
        ],
    },
    # package_dir={'': 'src'},
    # packages=setuptools.find_packages(where='src'),
    # packages=find_packages(exclude=('configs', 'tools', 'demo')),
    # package_dir={'chipeak_data_tool': 'chipeak_data_tool'},
    # packages=setuptools.find_packages(include=['chipeak_data_tool.*']),
    # python_requires='>=3.7',
)



# -*- coding: utf-8 -*-
# @Time : 2022/3/25 13:43
# @Author : Zhan Yong
from .base_labelme import BaseLabelme
from .label_infos import LabelInfo
from .total_labelme_infos import TotalLabelmeInfos

__all__ = ['BaseLabelme', 'LabelInfo', 'TotalLabelmeInfos']
